@@ -1,8 +1,9 @@
 #!/usr/bin/perl
-#$Id: statlib.pm 966 2012-05-25 18:29:30Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/statlib.pm $
+#$Id: statlib.pm 998 2013-08-14 12:21:20Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/statlib.pm $
 package    #hide from cpan
   statlib;
 use strict;
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 use Time::HiRes qw(time sleep);
 use utf8;
 our $root_path;
@@ -61,7 +62,7 @@ $config{'browser_ie'} = 1 if $ENV{'HTTP_USER_AGENT'} =~ /MSIE/ and $ENV{'HTTP_US
 #$config{'client'} = $_,
 $config{ 'browser_' . $_ } = 1 for grep { $ENV{'HTTP_USER_AGENT'} =~ /$_/i } @{ $config{'browsers'} };
 $config{'use_graph'} ||= 1;    #  if grep {$config{'browser_'. $_}} qw(firefox safari chrome opera);
-$config{'graph_inner'}        ||= 1 if grep { $config{ 'browser_' . $_ } } qw(firefox safari chrome);
+$config{'graph_inner'}        ||= 1 if grep { $config{ 'browser_' . $_ } } qw(firefox opera ); #  chrome safari 
 $config{'title'}              ||= 'dcstat';
 $config{'web_max_query_time'} ||= 10;
 #warn 'pre',Dumper $config{'sql'};
