@@ -1,8 +1,11 @@
-#$Id: DirectConnect.pm 966 2012-05-25 18:29:30Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
+#$Id: DirectConnect.pm 1002 2014-08-27 14:35:23Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
 package Net::DirectConnect;
 use strict;
-our $VERSION = '0.13';    # . '_' . ( split ' ', '$Revision: 966 $' )[1];
+no strict qw(refs);
+use warnings "NONFATAL" => "all";
 no warnings qw(uninitialized);
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
+our $VERSION = '0.14'; # . '_' . ( split ' ', '$Revision: 1002 $' )[1];
 use utf8;
 use Encode;
 use Socket;
@@ -17,7 +20,8 @@ our $AUTOLOAD;
 our %global;
 sub is_code ($) { UNIVERSAL::isa( $_[0], 'CODE' ) }
 sub code_run ($;@) { my $f = shift; return $f->(@_) if is_code $f }
-sub can_run ($$;@) { my $c = shift || return; my $f = shift || return; my $r = $c->can($f); return $r->( $c, @_ ) if $r; }
+sub is_object ($) { ref $_[0] and ref $_[0] ne 'HASH' }
+sub can_run ($$;@) { my $c = shift || return; return unless is_object $c; my $f = shift || return; my $r = $c->can($f); return $r->( $c, @_ ) if $r; }
 
 sub float {    #v1
   my $self = shift if ref $_[0];
@@ -70,15 +74,33 @@ sub send_udp ($$;@) {
 
 sub socket_addr ($) {
   my ($socket) = @_;
+  return wantarray ? ( $socket->peerhost, $socket->peerport ) : $socket->peerhost;
+
+=old  
   local @_;
-  eval { @_ = unpack_sockaddr_in( getpeername($socket) || return ) };
+  #eval { @_ = unpack_sockaddr_in( getpeername($socket) || return ) };
+  eval { @_ = unpack_sockaddr_in( $socket->peername || return ) };
   return unless $_[1];
   return unless $_[1] = inet_ntoa( $_[1] );
   return @_;
+=cut
+
+=todo
+  my ($err, $hostname, $servicename) = Socket::getnameinfo($socket->peername);
+if ($err) {
+  if (use_try 'Socket6') {
+  #warn "Cannot getnameinfo - $err; $hostname, $servicename" ;
+  $hostname = Socket6::getnameinfo($socket->peername);
+  }
+}
+warn 'getn', $hostname,$servicename;
+  return wantarray ? ($hostname,$servicename) : $hostname;
+=cut
+
 }
 
 sub schedule($$;@)
-{    #$Id: DirectConnect.pm 966 2012-05-25 18:29:30Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
+{    #$Id: DirectConnect.pm 1002 2014-08-27 14:35:23Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
   our %schedule;
   my ( $every, $func ) = ( shift, shift );
   my $p;
@@ -137,16 +159,18 @@ sub module_load {
 }
 
 sub new {
-  #print Dumper \@_;
+  #print 'NEW:',Dumper \@_;
   my $class = shift;
   my $self  = {};
   if ( ref $class eq __PACKAGE__ ) { $self = $class; }
   else                             { bless( $self, $class ) unless ref $class; }
   #print ref $self;
   #$self-
+  #psmisc::printlog('dev', 'new', @_);
   #psmisc::printlog('dev', 'func', Dumper @_);
   $self->{'number'} ||= ++$global{'total'};
   ++$global{'count'};
+  #$self->log('dev', 'new', @_);
   $self->func(@_);    #@param
   eval { $self->{'recv_flags'} = MSG_DONTWAIT; } unless $^O =~ /win/i;
   $self->{'recv_flags'} ||= 0;
@@ -163,23 +187,32 @@ sub new {
 #$self->{'parent'}{$_} ||= {} ,  $self->{$_} ||= $self->{'parent'}{$_},
 #$self->log( 'dev', '2uphandler my=',$self->{handler},Dumper($self->{handler}) , 'p=',Dumper($self->{'parent'}{handler}),$self->{'parent'}{handler},);
 #$self->log( 'dev', "my number=$self->{'number'} total=$global{'total'} count=$global{'count'}" );
+#$self->log( 'dev', $class ,' eq ', __PACKAGE__);
   if ( $class eq __PACKAGE__ ) {
-    #local %_ = (@param);
+    #local %_ = (@_);
+#$self->log( 'dev', $class ,' eq ', __PACKAGE__, Dumper @_);
+
     #for keys
     #$self->{$_} = $_{$_} for keys %_;
     #$self->log( 'init00', $self, "h=$self->{'host'}", 'p=', $self->{'protocol'}, 'm=', $self->{'module'} );
-    if ( $self->{'host'} eq 'broadcast' or $self->{'host'} =~ /^255\./ ) {
+    if ( $self->{'host'} ~~ m{^(?:\w+://)?broadcast} or $self->{'host'} =~ /^(?:255\.|\[?ff)/i ) {
+      #if (use_try 'Socket::Multicast6', 'ipv6') {
+        #IPV6_JOIN_GROUP
+      #}
       $self->{'protocol'} ||= 'adc';
       $self->{'auto_listen'}                 = 1;
+      delete $self->{'auto_connect'};
       $self->{'Proto'}                       = 'udp';
       $self->{'socket_options'}{'Broadcast'} = 1;
       $self->{'socket_options'}{'ReuseAddr'} = 1;
-      $self->{'host'}                        = inet_ntoa(INADDR_BROADCAST)
-        if $self->{'host'} !~ /^255\./;
+      #$self->{'host'} = $self->{dev_ipv6} ? 'ff02::1' : inet_ntoa(INADDR_BROADCAST) if $self->{'host'} !~ /^(?:255\.|\[?ff)/i;
+      $self->{'host'} = inet_ntoa(INADDR_BROADCAST) if $self->{'host'} !~ /^(?:255\.|\[?ff)/i;
+      #$self->{socket_options_listen}{'LocalHost'} = 'ff02::1';
       #$self->{'port'},
       #$self->log( 'dev',  "send to", );
       $self->{'broadcast'} = 1;
       #$self->{'lis'} = 1;
+      #$self->log('dev', "broadcast=$self->{'host'}, auto_listen=$self->{'auto_listen'} auto_connect=$self->{'auto_connect'}");
     }
     if (
       #!$self->{'module'} and
@@ -192,7 +225,7 @@ sub new {
       $self->{'protocol'} = $p;
       $self->{'protocol'} = 'nmdc'
         if !$self->{'protocol'}
-          or $self->{'protocol'} eq 'dchub';
+        or $self->{'protocol'} eq 'dchub';
       #$self->{'protocol'}
       #$self->log( 'proto ', $self->{'protocol'} );
     }
@@ -204,9 +237,29 @@ sub new {
     if ( $self->{'protocol'} eq 'nmdc' ) {
       ++$self->{'module'}{ $self->{'hub'} ? 'hubcli' : 'clihub' };
     }
-    ++$self->{'module'}{$_} for grep { $self->{ 'dev_' . $_ } } qw(ipv6 sctp);
+    #++$self->{'module'}{$_} for grep { $self->{ 'dev_' . $_ } } qw(ipv6);
+    #, ($] < 5.014 ? 'ipv6' : ()); #sctp
     #if ( $self->{'module'} ) {
   }
+  if ( $self->{'dev_sctp'} ) {
+    unless ( $self->{'no_sctp_fallback'} ) {
+      $self->log( 'dev', 'make sctp clone', $class );
+      $self->{'clients'}{ 'sctp_' . $self->{'number'} } = $class->new(
+        @_,
+        'dev_sctp'      => undef,
+        'parent'        => $self,
+        auto_work       => 0,
+        no_wait_connect => 1,
+        #reconnect_tries=>1,
+        #myport_tries=>1,
+        #'Proto'       => 'sctp',
+        modules => [qw(sctp)],
+      );
+    } else {
+      ++$self->{'module'}{'sctp'};
+    }
+    $self->info;
+  }
   ++$self->{'module'}{$_} for grep { $self->{'protocol'} eq $_ } qw(adcs http);
   #$self->log( 'dev', 'module load', $self->{'module'}, 'p', $self->{'protocol'} );
   my @modules;    #= ($self->{'module'});
@@ -216,8 +269,8 @@ sub new {
     push @modules, split /[;,\s]/, $self->{$_} unless ref $self->{$_};
   }
   #$self->log( 'dev', 'modules load', @modules );
-  #$self->log( 'modules load', @modules);
-  $self->module_load($_) for @modules;
+  #$self->log( 'modules load', @modules, @_);
+  $self->module_load( $_, @_ ) for @modules;
   #$self->log( 'now proto', $self->{'Proto'});
   $self->{charset_chat} ||= $self->{charset_protocol};
   #$self->protocol_init();
@@ -227,9 +280,9 @@ sub new {
   #eval qq{use encoding $self->{charset_console}};
   eval qq{use encoding '$self->{charset_internal}', STDOUT=> '$self->{charset_console}', STDIN => '$self->{charset_console}'}
     if !$self->{parent}
-      and !$self->{no_charset_console}
-      and $self->{charset_internal}
-      and $self->{charset_console};
+    and !$self->{no_charset_console}
+    and $self->{charset_internal}
+    and $self->{charset_console};
   #$self->log( 'dev', 'utf8: УТф восемь');
   #$self->log( 'dev', Dumper $self);
   $self->log( 'dcdbg',
@@ -246,16 +299,18 @@ sub new {
     }
   }
   #$self->log('dev', 'sctp', $self->{'dev_sctp'});
+  #$self->log('dev', "auto_listen=$self->{'auto_listen'} auto_connect=$self->{'auto_connect'}");
   if ( $self->{'auto_listen'} ) {
     #$self->{'disconnect_recursive'} = $self->{'parent'}{'disconnect_recursive'};
     $self->{'incomingclass'} ||= $self->{'parent'}{'incomingclass'};    # if $self->{'parent'};
     $self->listen();
+    #$self->log('go conaft');
     $self->connect_aft() if $self->{'broadcast'};
   } elsif ( $self->{'auto_connect'} ) {
     #$self->log( $self, 'new inited', "auto_connect MT:$self->{'message_type'}", ' with' );
     $self->connect();
     #$self->work();
-    $self->wait_connect();
+    $self->wait_connect() unless $self->{'no_wait_connect'};
   } else {
     $self->get_my_addr();
     $self->get_peer_addr();
@@ -311,14 +366,14 @@ sub cmd {
   }
   $self->{'last_cmd_time'} = time;
   $self->handler( $cmd . $handler . '_bef', \@_ );
-  #$self->{'log'}->($self,'dev', $self->{number},'cmdrun', $cmd, @_, $func) if $cmd ne 'log';
+  #$self->{'log'}->($self,'dev', $self->{number},'cmdrun', $dst, $cmd, @_, $func) if $cmd ne 'log';
   if ($func) {
     @ret = $func->( $self, @_ );    #$self->{'cmd'}{$cmd}->(@_);
+  } elsif ( $self->{'adc'} and length $dst == 1 and length $cmd == 3 ) {
+    @ret = $self->cmd_adc( $dst, $cmd, @_ );
   } elsif ( exists $self->{$cmd} ) {
     $self->log( 'dev', "cmd call by var name $cmd=$self->{$cmd}" );
     @ret = ( $self->{$cmd} );
-  } elsif ( $self->{'adc'} and length $dst == 1 and length $cmd == 3 ) {
-    @ret = $self->cmd_adc( $dst, $cmd, @_ );
   } else {
     $self->log(
       'dev',
@@ -366,7 +421,7 @@ sub handler {
   #$self->log('dev', 'handler select', $cmd, $self->{'handler_int'}{$cmd}, $self->{'handler'}{$cmd});
   $self->{'handler_int'}{$cmd}->( $self, @_ )
     if $self->{'handler_int'}
-      and ref $self->{'handler_int'}{$cmd} eq 'CODE';    #internal lib
+    and ref $self->{'handler_int'}{$cmd} eq 'CODE';    #internal lib
   $self->{'handler'}{$cmd}->( $self, @_ )
     if $self->{'handler'} and ref $self->{'handler'}{$cmd} eq 'CODE';
 }
@@ -450,17 +505,20 @@ sub init_main {    #$self->{'init_main'} ||= sub {
     #number => ++$global{'total'},
     #};
     'charset_fs' => (
-        $^O eq 'MSWin32' ? 'cp1251'
-      : $^O eq 'freebsd' ? 'koi8r'
+      $^O eq 'MSWin32'
+      ? 'cp1251'
+        #: $^O eq 'freebsd' ? 'koi8r'
       : 'utf8'
     ),
-    'charset_console' => ( $^O eq 'MSWin32' ? 'cp866' : $^O eq 'freebsd' ? 'koi8r' : 'utf8' ),
+    'charset_console' => ( $^O eq 'MSWin32' ? 'cp866' : 
+        #$^O eq 'freebsd' ? 'koi8r' : 
+        'utf8' ),
     'charset_protocol' => 'utf8',
     'charset_internal' => 'utf8',
     #charset_nick => 'utf8',
-    'socket_class' => 'IO::Socket::INET',
+    'socket_class' => ( use_try('IO::Socket::IP') ? 'IO::Socket::IP' : 'IO::Socket::INET' ),
   );
-  #$self->log(__LINE__, "Proto=$self->{Proto}");
+  #$self->log(__LINE__, "Proto=$self->{Proto}, protocol=$self->{protocol} class $self->{'socket_class'}");
   $self->{'wait_connect_tries'} //= $self->{'Timeout'};
   $self->{$_} //= $self->{'parent'}{$_} ||= {} for qw(peers peers_sid peers_cid handler clients);    #want share_full share_tth
          #$self->{$_} ||= $self->{'parent'}{$_} ||= {}, for qw(   );
@@ -469,11 +527,14 @@ sub init_main {    #$self->{'init_main'} ||= sub {
     for qw(sockets share_full share_tth want want_download want_download_filename downloading);
   $self->{$_} //= $self->{'parent'}{$_} ||= $global{$_}, for qw(db);
   $self->{'parent'}{$_} ? $self->{$_} //= $self->{'parent'}{$_} : ()
-    for qw(log disconnect_recursive  partial_prefix partial_ext download_to Proto dev_ipv6 socket_class protocol myport_inc)
-    ;   #dev_adcs
+    for
+    qw(log disconnect_recursive  partial_prefix partial_ext download_to Proto dev_ipv6 protocol myport_inc no_sctp_fallback)
+    ;   #dev_adcs socket_class 
         #$self->log( 'dev', "Proto=$self->{Proto}, Listen=$self->{Listen} protocol=$self->{protocol} inc=$self->{myport_inc}" );
+  $self->{$_} //= { %{ $self->{'parent'}{$_} } } for qw(socket_options);    # clone, childs can change
   $self->{$_} //= $_{$_} for keys %_;
   $self->{'partial_prefix'} //= $self->{'download_to'} . 'Incomplete/';
+  #$self->log(__LINE__, "Proto=$self->{Proto}, protocol=$self->{protocol} class $self->{'socket_class'} ");
   #$self->log("charset_console=$self->{charset_console} charset_fs=$self->{charset_fs}");
   #psmisc::printlog('dev', 'init0', Dumper $self);
 }
@@ -483,7 +544,7 @@ sub myport_generate {    #$self->{'myport_generate'} ||= sub {
   #$self->log( 'myport', "$self->{'myport'}: $self->{'myport_base'} or $self->{'myport_random'}" );
   return $self->{'myport'}
     unless $self->{'myport_base'}
-      or $self->{'myport_random'};
+    or $self->{'myport_random'};
   $self->{'myport'} = undef if $_[0];
   return $self->{'myport'} ||= $self->{'myport_base'} + $self->{'myport_inc'}++ if $self->{'myport_inc'};
   return $self->{'myport'} ||= $self->{'myport_base'} + int( rand( $self->{'myport_random'} ) );
@@ -508,13 +569,13 @@ sub connect_check {
   #$self->log('dev', 'connect_check', " s=$self->{'status'}, i=$self->{'incoming'}");
   return 0
     if $self->{'Proto'} eq 'udp'
-      or $self->{'incoming'}
-      #or $self->{'status'} eq 'listening' or
-      or $self->{'listener'}
-      or (
-        $self->{'socket'}
-        #and $self->{'socket'}->connected()
-      ) or !$self->active();
+    or $self->{'incoming'}
+    #or $self->{'status'} eq 'listening' or
+    or $self->{'listener'}
+    or (
+    $self->{'socket'}
+    #and $self->{'socket'}->connected()
+    ) or !$self->active();
 #$self->log(          'warn', 'connect_check: must reconnect', Dumper $self->{'socket'}->connected(), $self->{'socket'}, $self->{'status'});
   $self->{'status'} = 'reconnecting';
   $self->every(
@@ -523,7 +584,7 @@ sub connect_check {
       if ( $self->{'reconnect_tries'}++ < $self->{'reconnects'} ) {
         $self->log(
           'warn',
-          "reconnecting [$self->{'reconnect_tries'}/$self->{'reconnects'}] every",
+          "reconnecting ($self->{'host'}) [$self->{'reconnect_tries'}/$self->{'reconnects'}] every",
           $self->{'reconnect_sleep'}
         );
         $self->connect();
@@ -550,12 +611,12 @@ sub connect {    #$self->{'connect'} ||= sub {
   }
   #$self->log('dev', 'host, port =', $self->{'host'}, $self->{'port'} );
   #$self->log( 'H:', ((),$self->{'host'} =~ /(:)/g)>1 );
-  $self->module_load('ipv6') if ( @{ [ $self->{'host'} =~ /(:)/g ] } > 1 );
+  #$self->module_load('ipv6') if @{ [ $self->{'host'} =~ /(:)/g ] } > 1;
   #$self->{'port'} = $_[1] if $_[1];
   #print "Hhohohhhh" ,$self->{'protocol'},$self->{'host'};
   return 0
     if ( $self->{'socket'} and $self->{'socket'}->connected() )
-    or grep { $self->{'status'} eq $_ } qw(destroy);                                                 #connected
+    or grep { $self->{'status'} eq $_ } qw(destroy);    #connected
   $self->log(
     'info',
     "connecting to $self->{'protocol'}://[$self->{'host'}]:$self->{'port'} via $self->{'Proto'} class $self->{'socket_class'}",
@@ -567,20 +628,23 @@ sub connect {    #$self->{'connect'} ||= sub {
   #$self->{'port'}     = $1 if $self->{'host'} =~ s/:(\d+)//;
   delete $self->{'recv_buf'};
   #$self->log('dev', 'conn strt', $self->{'Timeout'}, $self->{'Proto'}, Socket::SOCK_STREAM);
-  $self->{'socket'} ||= $self->{'socket_class'}->new(
-    'PeerAddr' => $self->{'host'},
-    ( $self->{'port'}  ? ( 'PeerPort' => $self->{'port'} )  : () ),
-    ( $self->{'Proto'} ? ( 'Proto'    => $self->{'Proto'} ) : () ),
-    #( $self->{'Proto'} eq 'sctp' ? ( 'Type' => Socket::SOCK_STREAM ) : () ),
-    #'Timeout'  => $self->{'Timeout'},
-    #(
-    #$self->{'nonblocking'} ? (
-    'Blocking'   => 0,
-    'MultiHomed' => 1,    #del
-                          #) : ()
-                          #),
-    %{ $self->{'socket_options'} || {} },
-  );
+  eval {
+    $self->{'socket'} ||= $self->{'socket_class'}->new(
+      'PeerAddr' => $self->{'host'},
+      ( $self->{'port'}  ? ( 'PeerPort' => $self->{'port'} )  : () ),
+      ( $self->{'Proto'} ? ( 'Proto'    => $self->{'Proto'} ) : () ),
+      #( $self->{'Proto'} eq 'sctp' ? ( 'Type' => Socket::SOCK_STREAM ) : () ),
+      #'Timeout'  => $self->{'Timeout'},
+      #(
+      #$self->{'nonblocking'} ? (
+      'Blocking'   => 0,
+      #'MultiHomed' => 1,    #del
+                            #) : ()
+                            #),
+      %{ $self->{'socket_options'} },
+      %{ $self->{'socket_options_connect'} },
+    );
+  };
   #$self->log('dev', 'connect end');
   $self->log(
     'err',
@@ -636,8 +700,8 @@ sub connected {    #$self->{'connected'} ||= sub {
   $self->log( 'info', "my internal ip detected, using passive mode", $self->{'myip'}, $self->{'hostip'}, $localmask ),
     $self->{'M'} = 'P'
     if !$self->{'M'}
-      and $is_local_ip->( $self->{'myip'} )
-      and !$is_local_ip->( $self->{'hostip'} );
+    and $is_local_ip->( $self->{'myip'} )
+    and !$is_local_ip->( $self->{'hostip'} );
   $self->{'M'} ||= 'A';
   #$self->log( 'info', "mode set [$self->{'M'}] ");
   $self->log( 'info', "connect to $self->{'host'}($self->{'hostip'}):$self->{'port'} [me=$self->{'myip'}] ok ", );
@@ -655,16 +719,16 @@ sub reconnect {    #$self->{'reconnect'} ||= sub {
   #!$self->connect();
 }
 
-sub listen {       #$self->{'listen'} ||= sub {
+sub listen {    #$self->{'listen'} ||= sub {
   my $self = shift;
   $self->log( 'err', 'listen off', "[$self->{'Listen'}] [$self->{'M'}] [$self->{'allow_passive_ConnectToMe'}]" ), return
     if !$self->{'Listen'};
   #or ( $self->{'M'} eq 'P' and !$self->{'allow_passive_ConnectToMe'} );    #RENAME
   $self->{'listener'} = 1;
   $self->myport_generate();
-#$self->log( 'dev', 'listen', "p=$self->{'myport'}; proto=$self->{'Proto'} cl=$self->{'socket_class'}",'sockopts', Dumper $self->{'socket_options'} );
+  #$self->log( 'dev', 'listen', "p=$self->{'myport'}; proto=$self->{'Proto'} cl=$self->{'socket_class'}",'sockopts', Dumper $self->{'socket_options'}, $self->{'socket_options_listen'} );
   for ( 1 .. $self->{'myport_tries'} ) {
-    $self->{'socket'} ||= $self->{'socket_class'}->new(
+    local @_ = (
       'LocalPort' => $self->{'myport'},
       #'Proto'     => $self->{'Proto'} || 'tcp',
       ( $self->{'Proto'} ? ( 'Proto' => $self->{'Proto'} ) : () ),
@@ -675,16 +739,21 @@ sub listen {       #$self->{'listen'} ||= sub {
       ),
       #( $self->{'Proto'} eq 'sctp' ? ( 'Type' => Socket::SOCK_STREAM ) : () ),
       #( $self->{'nonblocking'} ? ( 'Blocking' => 0 ) : () ),
-      'Blocking' => 0,
-      #SSL_server=>1,
-      %{ $self->{'socket_options'} || {} },
+      Blocking  => 0,
+      #ReuseAddr => 1,
+      %{ $self->{'socket_options'} },
+      %{ $self->{'socket_options_listen'} },
     );
+    #$self->log( 'dev', 'listen', $self->{'socket_class'}, @_);
+    eval { $self->{'socket'} ||= $self->{'socket_class'}->new(@_); };
     $self->select_add(), last if $self->{'socket'};
-    $self->log( 'err', "listen($self->{'Listen'}) $self->{'myport'} socket error: $@" ), $self->myport_generate(1),
+    $self->log( 'err', "listen [$_/$self->{'myport_tries'}] ($self->{'Listen'}) $self->{'myport'} socket error: $@" ),
+      $self->myport_generate(1),
       unless $self->{'socket'};
   }
   $self->log( 'err', 'cant listen' ), return unless $self->{'socket'};
-  $self->log( 'info', "listening $self->{'myport'} $self->{'Proto'} with $self->{'socket_class'}" );
+  eval { $self->{'listener'} = $self->{'socket'}->sockhost; };
+  $self->log( 'info', "listening", $self->{'listener'}, "$self->{'myport'} $self->{'Proto'} with $self->{'socket_class'}" );
   $self->{'accept'} = 1 if $self->{'Proto'} ne 'udp';
   $self->{'status'} = 'listening';
   #$self->select();
@@ -747,7 +816,7 @@ sub destroy {
   #!?%$self = ();
 }
 
-sub recv {                # $self->{'recv'} ||= sub {
+sub recv {    # $self->{'recv'} ||= sub {
   my $self   = shift;
   my $client = shift;
   #my $socket = shift;
@@ -757,15 +826,22 @@ sub recv {                # $self->{'recv'} ||= sub {
     #and $client eq $self->{'socket'}
     )
   {
-    if ( local $_ = $self->{'socket'}->accept() ) {
+    local $_ = $self->{'socket'}->accept();
+    if ($_) {
       #$self->log( 'traceDEV', 'DC::recv', 'accept', $self->{'incomingclass'} );
       $self->log( 'err', 'cant accept, no incomingclass' ), return,
         unless $self->{'incomingclass'};
-      my ( undef, $host ) = socket_addr $_;
-      ;    # || $self->{parent}{'allow'};
-      $self->log( 'info', "incoming [$host] to $self->{'incomingclass'}" );
+      #my $host = $_->peerhost;
+      my ($host) = socket_addr $_;
+#;    # || $self->{parent}{'allow'};
+#$self->log( 'info', "incoming [$host] (".ref($_).")to $self->{'incomingclass'}", ($self->{'allow'} ? "allow=$self->{'allow'}" : ()) );
+      $self->log(
+        'info',
+        "incoming [$host] (" . ref($_) . ") to $self->{'incomingclass'}",
+        ( $self->{'allow'} ? "allow=$self->{'allow'}" : () )
+      );
       if ( my $allow = $self->{'allow'} ) {
-        my ( undef, $host ) = socket_addr $_;
+        #my ( undef, $host ) = socket_addr $_;
         $self->log( 'warn', "disallowed connect from $host" ), return
           unless $host eq $allow;
       }
@@ -793,7 +869,7 @@ sub recv {                # $self->{'recv'} ||= sub {
       #$self->log( 'dev', 'child created',            $_,   $self->{'clients'}{$_});
       #++$ret;
     } else {
-      $self->log( 'err', "Accepting fail! [$self->{'Proto'}]" );
+      $self->log( 'err', "Accepting fail! ($_) [$self->{'Proto'}]", $!, $@ );
     }
     #next;
     return;
@@ -803,8 +879,8 @@ sub recv {                # $self->{'recv'} ||= sub {
   $self->{'databuf'} = '';
   #my $r;
   my $recv = $self->{'recv'};
-  if ( !defined( $self->{'recv_addr'} = $client->$recv( $self->{'databuf'}, POSIX::BUFSIZ, $self->{'recv_flags'} ) )
-    or !length( $self->{'databuf'} ) )
+  if ( (!defined( $self->{'recv_addr'} = $client->$recv( $self->{'databuf'}, POSIX::BUFSIZ, $self->{'recv_flags'} ) )
+    or !length( $self->{'databuf'} )) and $recv ~~ 'recv' )
   {
     #TODO not here
     if (
@@ -838,7 +914,7 @@ sub recv {                # $self->{'recv'} ||= sub {
     #$self->log( 'dcdmp', "rawrawbuf:", $self->{'recv_buf'} );
     local $self->{'cmd_aft'} = "\x0A"
       if !$self->{'adc'}
-        and $self->{'recv_buf'} =~ /^[BCDEFHITU][A-Z]{,5} /;
+      and $self->{'recv_buf'} =~ /^[BCDEFHITU][A-Z]{,5} /;
 #$self->log( 'dcdbg', "[$self->{'number'}]", "raw to parse [$self->{'buf'}] sep[$self->{'cmd_aft'}]" ) unless $self->{'filehandle'};
     $self->{'recv_buf'} .= $self->{'cmd_aft'}
       if $self->{'Proto'} eq 'udp' and $self->{'status'} eq 'listening';
@@ -858,7 +934,7 @@ sub recv {                # $self->{'recv'} ||= sub {
     }
     $self->file_write( \$self->{'recv_buf'} ), $self->{'recv_buf'} = ''
       if length( $self->{'recv_buf'} )
-        and $self->{'filehandle'};
+      and $self->{'filehandle'};
   }
 }
 
@@ -902,13 +978,15 @@ sub select {    #$self->{'select'} ||= sub {
       if $self->{sockets}{$_}{status} eq 'connecting_tcp' and $self->{sockets}{$_}{socket}->connected();
   }
 =cut
-
   for (@$send) {
     next unless $self->{sockets}{$_} and $self->{sockets}{$_}{socket};
+    #$self->log('connect test', $self->{sockets}{$_}{status}, $self->{sockets}{$_}{socket}->connected(), caller);
+    #$self->{'select'}->remove($_),
     $self->{sockets}{$_}->connected(),
       #can_run($self->{sockets}{$_}, 'connected'),
       ++$ret,
-      if $self->{sockets}{$_}{status} eq 'connecting_tcp' and $self->{sockets}{$_}{socket}->connected();
+      if $self->{sockets}{$_}{status} eq 'connecting_tcp';
+    # and $self->{sockets}{$_}{socket}->connected();
     #$self->log( 'err', 'no object for send handle',$_,  ) , next , unless $self->{sockets}{$_};
     #++$self->{sockets}{$_}{send_can};
     #$self->log( 'dev', 'can_send', $_, $self->{sockets}{$_}{number}, $self->{sockets}{$_}{send_can} );
@@ -963,11 +1041,11 @@ sub finished {    #$self->{'finished'} ||= sub {
   local @_;
   $self->log( 'dcdev', 'not finished clients:', @_ ), return 0
     if @_ =
-      grep { !$self->{'clients'}{$_}->finished() } $self->clients_my();    #keys %{ $self->{'clients'} };
+    grep { !$self->{'clients'}{$_}->finished() } $self->clients_my();    #keys %{ $self->{'clients'} };
   return 1;
 }
 
-sub wait_connect {                                                         #$self->{'wait_connect'} ||= sub {
+sub wait_connect {                                                       #$self->{'wait_connect'} ||= sub {
   my $self = shift;
   #$self->log( 'dev', "wait_connect", $self->{'wait_connect_tries'});
   for ( 0 .. ( $_[0] || $self->{'wait_connect_tries'} ) ) {
@@ -979,7 +1057,7 @@ sub wait_connect {                                                         #$sel
   return $self->{'status'};
 }
 
-sub wait_finish {                                                          #$self->{'wait_finish'} ||= sub {
+sub wait_finish {                                                        #$self->{'wait_finish'} ||= sub {
   my $self = shift;
   my $time = time() + ( shift || $self->{'wait_finish'} );
   while ( $time > time() ) {
@@ -1008,7 +1086,7 @@ sub wait_clients {                    #$self->{'wait_clients'} ||= sub {
     local @_;
     last
       if !$self->{'clients_max'}
-        or $self->{'clients_max'} > ( @_ = $self->clients_my() );    #keys %{ $self->{'clients'} };
+      or $self->{'clients_max'} > ( @_ = $self->clients_my() );    #keys %{ $self->{'clients'} };
     $self->info() unless $_;
     $self->log(
       'info', "wait clients",
@@ -1020,7 +1098,7 @@ sub wait_clients {                    #$self->{'wait_clients'} ||= sub {
   }
 }
 #sub wait_sleep {                                                     #$self->{'wait_sleep'} ||= sub {
-sub wait {                                                           #$self->{'wait_sleep'} ||= sub {
+sub wait {                                                         #$self->{'wait_sleep'} ||= sub {
   my $self = shift;
   my $ret;
   my $time = time() + ( shift || 1 );
@@ -1037,6 +1115,8 @@ sub wait {                                                           #$self->{'w
 
 sub work {    #$self->{'work'} ||= sub {
   my $self   = shift;
+  #$self->log( 'dev', 'work', ref $self->{parent}, $self->{parent});
+  return $self->{parent}->work(@_) if is_object($self->{parent});
   my @params = @_;
   #$self->periodic();
   #$self->log( 'dev', 'work', @params);
@@ -1088,6 +1168,7 @@ sub work {    #$self->{'work'} ||= sub {
         next if $self->{'sockets'}{$_} and %{ $self->{'sockets'}{$_} };
         delete $self->{'sockets'}{$_};
       }
+#$self->log('dev', 'parent:', $self->{parent}, $self->{parent}{parent}, is_object($self->{parent}));
       if ( !$self->{parent} or !$self->{parent}{parent} ) {    # first parent always autocreated on init
         code_run( $self->{$_}, $self ) for qw(worker);         #auto_work
       }
@@ -1143,7 +1224,7 @@ sub work {    #$self->{'work'} ||= sub {
           sort {
             $wdls->{$a}{tries} <=> $wdls->{$b}{tries}
               or ( $wdls->{$b}{slotsfree} || $wdls->{$b}{SL} ) <=> ( $wdls->{$a}{slotsfree} || $wdls->{$a}{SL} )
-            } keys %$wdls
+          } keys %$wdls
         );
         #$self->log('dev', 'from can:',     @_   );
         if ( my ($fromk) = $_[0] ) {
@@ -1202,11 +1283,14 @@ sub work {    #$self->{'work'} ||= sub {
   $self->select( 1 || $self->{'work_sleep'} );    # if @{$self->{send_buffer_raw}|| []};    # maybe send
                                                   #$self->log( 'dev', "work -> sleep", @params ),
   return $self->wait(@params) if @params;
-  return $self->select() || $self->select( $self->{'work_sleep'}, 1 );    # unless @{$self->{send_buffer_raw}|| []};
-                                                                          #return $self->select( $self->{'work_sleep'} );
+  return
+       $self->select( undef, 1 )
+    || $self->select( $self->{'work_sleep'} )
+    || $self->select( undef, 1 );                 # unless @{$self->{send_buffer_raw}|| []};
+                                                  #return $self->select( $self->{'work_sleep'} );
 }
 
-sub dumper {                                                              #$self->{'dumper'} ||= sub {
+sub dumper {                                      #$self->{'dumper'} ||= sub {
   my $self = shift;
   my $file = $_[0] || $self->{dev_auto_dump_file} || $0 . ( $self->{dev_auto_dump_timed} ? '.' . time : () ) . '.dump';
   open my $fh, '>', $file or return;
@@ -1215,7 +1299,7 @@ sub dumper {                                                              #$self
   $self->log( 'dev', "Writed dump", -s $file );
 }
 
-sub parser {                                                              #$self->{'parser'} ||= sub {
+sub parser {                                      #$self->{'parser'} ||= sub {
   my $self = shift;
   for ( local @_ = @_ ) {
     $self->log(
@@ -1233,8 +1317,8 @@ sub parser {                                                              #$self
     if (/^[<*]/) {
       $cmd = ( $self->{'status'} eq 'connected' ? 'chatline' : 'welcome' );
     }
-    s/^\$?([\w\-]+)\s*//, $cmd = $1 unless $cmd;
-    #$self->log('dev',"cmd[", Dumper($cmd),"]" );
+    s/^\x00*\$?([\w\-]+)\s*//, $cmd = $1 unless $cmd; # \x00 - ssl bug on recv
+    #$self->log('dev',"cmd[", Dumper($cmd),"], adc=", $self->{'adc'} );
     if ( $self->{'adc'} ) {
       $cmd =~ s/^([BCDEFHIU])//, $dst = $1;
       @param = ( [$dst], split / / );
@@ -1263,7 +1347,7 @@ sub parser {                                                              #$self
     #$self->log( 'dcdmp', "P3 $dst$cmd p=",(Dumper \@param));
     $cmd = $dst . $cmd
       if !exists $self->{'parse'}{$cmd}
-        and exists $self->{'parse'}{ $dst . $cmd };
+      and exists $self->{'parse'}{ $dst . $cmd };
     #$self->log( 'dcinf', "UNKNOWN PEERCMD:[$cmd]->($_) : please add \$dc->{'parse'}{'$cmd'} = sub { ... };" ),
     $self->{'parse'}{$cmd} = sub { }, $cmd = ( $self->{'status'} eq 'connected' ? 'chatline' : 'welcome' )
       if $self->{'nmdc'} and !exists $self->{'parse'}{$cmd};
@@ -1345,13 +1429,14 @@ sub send {    #$self->{'send'} ||= sub {
 	$self->{send_buffer_raw} = undef;
     }
 =cut
+
   #return unless @_;
 }
 
 sub sendcmd {    #$self->{'sendcmd'} ||= sub {
   my $self = shift;
   return if $self->connect_check();
-  return if $self->{'listener'};
+  return if $self->{'listener'} and !$self->{'broadcast'};
   #$self->{'log'}->( $self,'sendcmd0', @_);
   local @_ = @_, $_[0] .= splice @_, 1, 1
     if $self->{'adc'} and length $_[0] == 1;
@@ -1437,7 +1522,7 @@ sub get {                                                 #$self->{'get'} ||= su
     || $self->{peers}{$sid}
     || $self->{peers}{$nick}
     || {};
-  $self->log( 'dbg', "getting [$nick] $file as $as  sid=[$sid]:$self->{'myport'}" );    #, Dumper $peer->{INF});
+  $self->log( 'dbg', "getting [$nick] $file as $as sid=[$sid]:$self->{'myport'} p=$self->{'protocol_connect'}" );    #, Dumper $peer->{INF});
   if ( $self->{'adc'} ) {
     #my $token = $self->make_token($nick);
     local @_;
@@ -1657,7 +1742,7 @@ sub file_send {    #$self->{'file_send'} ||= sub {
   $size = $filesize - $start if $size <= 0;
   $self->log( 'dcerr', "cant find [$file]" ), $self->disconnect(), return
     if !-e $file
-      or -d $file;
+    or -d $file;
   if ( open $self->{'filehandle_send'}, '<', $file ) {
     binmode( $self->{'filehandle_send'} );
     seek( $self->{'filehandle_send'}, $start, SEEK_SET ) if $start;
@@ -1832,9 +1917,11 @@ sub get_peer_addr {    #$self->{'get_peer_addr'} ||= sub () {
   my $self = shift if ref $_[0];
   my ($recv) = @_;
   return unless $self->{'socket'};
+  eval {
   $self->{'port'}   = $self->{'socket'}->peerport();
   $self->{'hostip'} = $self->{'socket'}->peerhost();
   $self->{'host'} ||= $self->{'hostip'};
+  };
   return $self->{'hostip'};
 
 =no
@@ -1847,6 +1934,7 @@ sub get_peer_addr {    #$self->{'get_peer_addr'} ||= sub () {
     if $_[1];
   return $self->{'hostip'};
 =cut
+
 }
 
 sub get_peer_addr_recv {    #$self->{'get_peer_addr_recv'} ||= sub (;$) {
@@ -1882,7 +1970,6 @@ sub get_my_addr {           #$self->{'get_my_addr'} ||= sub {
   #$self->{'log'}->('dev', "MYIP($self->{'myip'}) [$self->{'number'}] SOCKNAME $_[0],$_[1];");
   return $self->{'myip'} ||= $_[1];
 =cut
-
 }
 
 sub info {    #$self->{'info'} ||= sub {
@@ -1953,8 +2040,8 @@ sub cmd_adc {            #$self->{'cmd_adc'} ||= sub {
   $self->sendcmd(
     $dst, $cmd,
     #map {ref $_ eq 'ARRAY' ? @$_:ref $_ eq 'HASH' ? each : $_)    }@_
-    (                    #$self->{'broadcast'} ? $self->{'INF'}{'SID'} #$self->{'INF'}{'ID'}
-                         #:
+    (    #$self->{'broadcast'} ? $self->{'INF'}{'SID'} #$self->{'INF'}{'ID'}
+          #:
       ( $dst eq 'C' or !length $self->{'INF'}{'SID'} )
       ? ()
       : $self->{'INF'}{'SID'}
@@ -2129,9 +2216,11 @@ look at examples for handlers
 
  To install this module type the following:
 
-   cpan DBD::SQLite IO::Socket::INET6 IO::Socket::SSL
+   cpan DBD::SQLite IO::Socket::IP IO::Socket::INET6 IO::Socket::SSL
    perl Makefile.PL && make install clean
 
+ debian:
+ apt-get install libdbd-sqlite3-perl libio-socket-ip-perl libjson-xs-perl libjson-perl libmime-base32-perl liblib-abs-perl
 
 =head1 SEE ALSO
 
