@@ -1,7 +1,11 @@
-#$Id: clihub.pm 966 2012-05-25 18:29:30Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
+#$Id: clihub.pm 998 2013-08-14 12:21:20Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
 package    #hide from cpan
   Net::DirectConnect::clihub;
 use strict;
+no strict qw(refs);
+use warnings "NONFATAL" => "all";
+no warnings qw(uninitialized);
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 use utf8;
 use Time::HiRes qw(time sleep);
 use Data::Dumper;    #dev only
@@ -9,8 +13,7 @@ $Data::Dumper::Sortkeys = $Data::Dumper::Indent = 1;
 use Net::DirectConnect;
 use Net::DirectConnect::clicli;
 #use Net::DirectConnect::http;
-no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 966 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 998 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub name_to_ip($) {
@@ -94,6 +97,7 @@ sub init {
           or $text =~ /Search ignored\.  Please leave at least (\d+) seconds between search attempts\./  #Hub-Security opendchub
           or $text =~
 /Минимальный интервал между поисковыми запросами:(\d+)сек., попробуйте чуть позже/
+          or $text =~ /You can do 1 searches in (\d+) seconds/
           )
         {
           $self->{'search_every'} = int( rand(5) + $1 || $self->{'search_every_min'} );
@@ -132,7 +136,8 @@ sub init {
           $self->{'Nick'} = $try if length $try;
         } elsif ( $text =~ /Bad nickname: Wait (\d+)sec before reconnecting/i
           or $text =~
-          /Пожалуйста подождите (\d+) секунд до повторного подключения\./ )
+          /Пожалуйста подождите (\d+) секунд до повторного подключения\./
+          or $text =~ /Do not reconnect too fast. Wait (\d+) secs before reconnecting./ )
         {
           #sleep $1 + 1;
           $self->work( $1 + 10 );
@@ -231,7 +236,8 @@ sub init {
       my ( $nick, $host, $port ) = $_[0] =~ /\s*(\S+)\s+(\S+)\:(\S+)/;
       $self->{'IpList'}{$host}{'port'} = $self->{'PortList'}->{$host} = $port;
       #$self->log('dev', "portlist: $host = $self->{'PortList'}->{$host} :=$port");
-      $self->log("ignore flooding attempt to [$host:$port ] ($self->{flood}{$host})"), $self->{flood}{$host} = time + 30, return
+      $self->log("ignore flooding attempt to [$host:$port ] ($self->{flood}{$host})"), $self->{flood}{$host} = time + 30,
+        return
         if $self->{flood}{$host} > time;
       $self->{flood}{$host} = time + 60;
       return if $self->{'clients'}{ $host . ':' . $port }->{'socket'};
@@ -364,7 +370,7 @@ sub init {
       ( $params->{'nick'}, $params->{'str'} ) = split / /, $_[0], 2;
       $params->{'str'} = [ split /\x05/, $params->{'str'} ];
       $params->{'file'} = shift @{ $params->{'str'} };
-      ( $params->{'filename'} ) = $params->{'file'}     =~ m{([^\\]+)$};
+      ( $params->{'filename'} ) = $params->{'file'} =~ m{([^\\]+)$};
       ( $params->{'ext'} )      = $params->{'filename'} =~ m{[^.]+\.([^.]+)$};
       ( $params->{'size'}, $params->{'slots'} )  = split / /, shift @{ $params->{'str'} };
       ( $params->{'tth'},  $params->{'ipport'} ) = split / /, shift @{ $params->{'str'} };
@@ -412,6 +418,7 @@ sub init {
 
 
 =cut  
+
   #$self->{'cmd'} = {
   local %_ = (
     'connect_aft' => sub {
@@ -633,6 +640,7 @@ sub init {
     $self->log( 'err', "cant listen http" )
       unless $self->{'myport_http'};
 =cut
+
   $self->{'handler_int'}{'disconnect_bef'} = sub {
     #delete $self->{'sid'};
     #$self->log( 'dev', 'disconnect int' ) if $self and $self->{'log'};
