@@ -1,11 +1,14 @@
-#$Id: hub.pm 593 2010-01-30 11:11:27Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hub.pm $
+#$Id: hub.pm 998 2013-08-14 12:21:20Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hub.pm $
 package    #hide from cpan
   Net::DirectConnect::hub;
 use Net::DirectConnect;
 use Net::DirectConnect::hubcli;
 use strict;
+no strict qw(refs);
+use warnings "NONFATAL" => "all";
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 593 $' ) )[1];
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
+our $VERSION = ( split( ' ', '$Revision: 998 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub init {
