@@ -1,11 +1,14 @@
-#$Id: hubhub.pm 505 2009-11-22 03:52:21Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hubhub.pm $
+#$Id: hubhub.pm 998 2013-08-14 12:21:20Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hubhub.pm $
 #reserved for future 8)
 package    #hide from cpan
   Net::DirectConnect::hubhub;
 use Net::DirectConnect;
 use strict;
+no strict qw(refs);
+use warnings "NONFATAL" => "all";
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 505 $' ) )[1];
+no if $] >= 5.017011, warnings => 'experimental::smartmatch';
+our $VERSION = ( split( ' ', '$Revision: 998 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub init {
